%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Pradeep Nagesh, Arizona State University, USA (2007-2008)  
%         pnagesh@asu.edu
%
% Description: 
%  Main file for Face Recognition using JSM Models. This file
%  invokes routines for performing training and testing.               
%  --Training: First, involves generating JSM data from training set i.e
%  using trainig set and specified number of images per class, 
%  stacked measument "y", stacked basis matrix (eg: using DCT) etc, 
%  are precomuted and stored as .mat files. Next, these files are used to 
%  perform training i.e solve the JSM CS optimization using specificed
%  setting/algorithm to generate training features - Common, Innovations
%  and Gross Innovation (sum of Innovations).
%
%  --Testing: Inolves classification of test images in folder using the
%  stored training features. Again parameters such as algorithm, basis
%  matrix etc can be specified.
%
%  Input/Output 
%  Options: Input data structure containing various parameter settings.  
%  Refer to FR_Document.doc for details.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function status = FaceTraining_mainFunction(Options)

%%       Set up the parameters for training and Perform Checks                    

% Check : Are we in right folder?? If not Quit
checkWord = 'JSM_Face_Recognition';
currentPath = pwd;
checkPath = currentPath(end-19:end);

if ~strcmp(checkPath,checkWord)
    error('Please cd to the correct directory before running the program... cd...\JSM_Face_Recognition');
end

% Global variables specifiying if some operations should be done/ or
% skipped if already done
global checkJSM_dataGenerated;
global checkJSM_TrainingDone;
global checkJSM_TestingDone;
global test_img_counter;
global ClassRes;

JSM_Data_Sample_Frac = Options.SampleFraction;

N1 = Options.SizeImage(1);
N2 = Options.SizeImage(2);
L = Options.L;
M = Options.M;

disp_opts = sprintf('The Options for the Face Recognition is set as\n'); disp(disp_opts);
disp(Options);

%%              Pre-training Process

% Generate all data required for training and store it in correct folders
% in the folder structure. This can be used later if training features need
% to be generated again and again for a given database, until the
% parameters 'image size' and 'number of images per class' are same.

if (checkJSM_dataGenerated)
    disp('The training is skipped since the variable checkJSM_dataGenerated is set to 1');
    disp('If new training is reequired, then set checkJSM_dataGenerated is set to 0');
    status = 0;
else
    disp_trn = sprintf('Generating JSM Data from trainig images in DataBase %s for the given options...\n', Options.DataBaseName); disp(disp_trn);
    disp_trn = sprintf('This might take some time...\n'); disp(disp_trn);
    for i =1:length(JSM_Data_Sample_Frac)
        generate_JSM_Face_data(Options,JSM_Data_Sample_Frac(i));
    end
    disp_trn = sprintf('Generated JSM data for the data base %s\n',Options.DataBaseName); disp(disp_trn);
end

%%                Start Training

% If training is required (generating features), then the routine FaceDatabase_Train_Class() is
% invoked. The parameters are passed in "Options" strucuture and training
% is done for each class (subject) from i=1....L

if checkJSM_TrainingDone
    disp('JSM Training is already done on current database.. We use the stored data..')
else
    disp_trn = sprintf('Training Started\n'); disp(disp_trn);
    for i =1:L
        status = FaceDatabase_Train_Class(Options,i);
    end

    if status
        disp_trn = sprintf('Training Finished..'); disp(disp_trn);
    end
end

%%                Start Testing
if (0 == checkJSM_TestingDone)
    
    TestData_RootPath = GetPath_name('Testing Data',Options);
    list_image_name = dir(strcat(TestData_RootPath,'*.jpg'));
    Size_FullTest_Set = length(list_image_name);

    disp_tst = sprintf('Testing on database %s begins...', Options.DataBaseName); disp(disp_tst);
    disp_tst = sprintf('Test Images in the folder %s', TestData_RootPath); disp(disp_tst);

    % set up for testing
    testOptions = Options;
    testOptions.ClassNum=1;
    testOptions.M = 2;      % M=2 since only 2 images are considered for testing 
                            % test image and feature image of one class at a time
                            
                            
    % Preload the test data such as basis matrix, measurement matrix. If
    % not already stored as mat files, then compute them and store it so that they can
    % be used in future for similar settings (eg: similar image size etc..)
    try
        load(GetPath_name('JSM Test Data',testOptions), 'idctmx');
    catch
        if (0 == Options.twoDReconstruction)
            [Phi shi_tilda]= Phi_Shi_TetaMat(testOptions);
            idctmx = khronical_prdct(dctmtx(N1)');
        else
            [Phi shi_tilda] = Phi_Shi_TetaMat2D(testOptions);
            idctmx = dctmtx(N1)';
        end
        tet = Phi * shi_tilda;
        testOptions.ClassNum = 1;
        save(GetPath_name('JSM Test Data',testOptions),'Phi', 'tet','idctmx', 'shi_tilda');
    end

    % Classificartion Results to be stored in this Array
    ClassRes = zeros(2,Size_FullTest_Set);
    
    % Loop over entire test folder and run the classifier
    for i=1:Size_FullTest_Set
        sprintf('test image number = %d',i)
        test_img_counter = i;
        testImage_name = strcat(TestData_RootPath,list_image_name(i).name);
        detected_class = FaceTestClass_Sparsity_Distance(testImage_name,Options);
    end
    disp_tst = sprintf('Testing on database %s completed...', Options.DataBaseName); disp(disp_tst);
end